// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// 
// Module: Arfgen
// Author: Hassan Siddiqui
//
// Description: Main class for generating ARF
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: Arfgen.cc,v 1.2 2014/04/21 07:45:57 aibarra Exp $

/* General */
#include <iostream>

/* SAS-specific */
#include "Xmm.h"
#include "param.h"
#include "Dal.h"
#include "Tracer.h"

/* Local */
#include "Arfgen.h"
#include "ArfDataServer.h"
#include "ArfTableIterator.h"
#include "SetBackScale.h"


void Arfgen::exec()
{
      
      PROC;
      
      // Open the arf dataset
      string arfsetname = stringParameter("arfset");
      errstr 
	<< "Opening arfset " << arfsetname << "..."
	<< Sas::message(Msg::App, Msg::Sparse);

      DataSet * arfset;

      if (booleanParameter("keeparfset")) {
         arfset = dataSetServer->
            open(stringParameter("arfset"), DataSet::Create);
      } else {
         arfset = dataSetServer->
            open(stringParameter("arfset"), DataSet::Temp);
      }
      //DataSet * arfset = dataSetServer->
   	   // open(stringParameter("arfset"), DataSet::Create);

      // Add the arf table to the dataset

      Table * arftable = arfset->addTable("SPECRESP", 
					  arfDataServer->ebins()->bins());

      // Add the attributes appropriate for an OGIP-compliant ARF
      
      *arftable->addAttribute("TELESCOP",    "mission/satellite name")                  = "XMM";
      
      *arftable->addAttribute("INSTRUME",    "instrument/detector name")                = Xmm::toString(arfDataServer->instrumentID());

      *arftable->addAttribute("FILTER",      "filter in use")                           = " ";
      
      *arftable->addAttribute("ARFVERSN",    "OGIP classification of FITS format")      = "1992a";
      
      *arftable->addAttribute("HDUCLASS",    "Format conforms to OGIP standard")        = "OGIP";
      
      *arftable->addAttribute("HDUCLAS1",    "dataset relates to spectral response")    = "RESPONSE";

      *arftable->addAttribute("HDUCLAS2",    "dataset is a spectral response matrix")   = "SPECRESP";

      *arftable->addAttribute("HDUVERS1",    "Obsolete - included for backwards compatibility")     = "1.0.0";

      *arftable->addAttribute("HDUVERS2",    "Obsolete - included for backwards compatibility")     = "1.1.0";

      *arftable->addAttribute("HDUVERS",    "the version of the HDUCLAS2 format in use")            = "1.1.0";

  
      // Set up the table iterator
      ArfTableIterator arftableiterator;

      // Run it
      arftableiterator.processEntireTable(arftable);

      // Clean up
      errstr 
	<< "Closing arfset " << arfsetname << "."
	<< Sas::message(Msg::App, Msg::Sparse);
      dataSetServer->close(arfset);

      /* Set the backscale value in the input spectrum if required */
      if (booleanParameter("setbackscale")) {
         SetBackScale();
      }
  
      ENDPROC;

    } 
