// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: DetmapPixelIterator
// Author: Richard Saxton
//
// Description: 
//
// Class representation of an iterator that handles DAL-based detectormaps.
// It translates between two different coordinate systems if required and returns
// detector pixels in the second system.
//
// Modifications: See ChangeLog
// 
// General classes
#include <iostream>
#include<cassert>
#include <limits>

// SAS-specific classes
#include "Dal.h"
#include "SscWcsConvertor.h"

// General utility functions
#include "errstr.h"
#include "Tracer.h"

// Local classes
#include "DetmapPixelIterator.h"

/* Constructors */

// arrayname: detector map array name
// spectrumname: name of the spectral file
DetmapPixelIterator::DetmapPixelIterator(const string & arrayname, 
                                                  const string & spectrumname) :
ArrayPixelIterator(arrayname)
{
 // Set up of base class ArrayPixelIterator done in constructor
 
 // The pointer to the array values is corrupted when the detector map is opened 
 // again to find the WCS values. To avoid this store them locally
  vector<double> valtemp;
  for (unsigned row=0; row<_size; row++)
      valtemp.push_back(valueAt(row));

 // Set the coord conversion matrices from the array
  SscWcsConvertor::setWcs(arrayname);

 // Set the attitude matrix if the array contains XMM detector coordinates
  if (_wcsconv->xType()=="DETX" || _wcsconv->xType()=="DETX_ANG") {
     SscWcsConvertor::setAtt(arrayname);
  }

 // Produce a vector containing the RA/DEC values from the input array iterator
  CoordPair det;
  vector<CoordPair> cel;

 // Initialise and iterate over the array elements
  first();
  while (! isDone())
    {
       // Translate from image pixel to RA/DEC
        cel.push_back(SscWcsConvertor::imagePixelsToRaDec(double(_xindex),
                                                             double(_yindex)));
        next();
    }

 // Now change coordinate conversion matrices to those of the spectrum
  SscWcsConvertor::setAtt(spectrumname);

 // Also change back the WCS ????
 
 // Convert from RA/DEC to DET coords in the coord system of the spectrum
 // and find min and max of detector coords
  _maxx=-100000000.0, _minx=100000000.0, _maxy=-100000000.0, _miny=100000000.0;
  int good_count=0;

  for (unsigned row=0; row<cel.size(); row++)
    {
      CoordPair det=SscWcsConvertor::raDecToDet(cel[row].x(), cel[row].y());      

      // Det coords eventually get converted to INT16 which can wrap around.
      // To avoid this exclude values which don't fit into an INT16 variable
      // which will be outside the XMM detectors.
      if ( det.x()>=std::numeric_limits<int16>::min() &&
          det.x()<=std::numeric_limits<int16>::max() &&
          det.y()>=std::numeric_limits<int16>::min() &&
          det.y()<=std::numeric_limits<int16>::max() )
      {
          _outDetCoord.push_back(det);

         // Find min and max of detector pairs 
          _minx=min(_minx,det.x()); _maxx=max(_maxx,det.x()); 
          _miny=min(_miny,det.y()); _maxy=max(_maxy,det.y()); 

         // Set values
          _valuev.push_back(valtemp[row]);     

          ++good_count;
      }
    }

 // Reset the size of the iterator
   if (good_count == 0) 
      errstr << "The detector map does not overlap the spectral extraction " 
             << "region" << Sas::error("BadDetectorMap");

   //cout<<"DBG: "<<cel[0].x()<<" "<<cel[0].y()<<" "<<_outDetCoord[0].x()<<" "<<_outDetCoord[0].y()<<" "<<good_count<<endl;

   _size=good_count;

 // If the width was originally in degrees then we want to change it into units of
 // detector coordinates. Options are:
 // 1. DET coords (PN) - do nothing
 // 2. DET_ANG coords (MOS) - do nothing because already converted in WCSCONVERTOR
 // 3. RA---TAN (either) - needs converting
 
 // make a new WCS convertor
    if (_wcsconv->xType()=="RA---TAN") {
     
       double deg2pix = 72000.0;  // Convert degrees to detector pixels
       WCSConvertor *newwcsconv = new WCSConvertor(_wcsconv->xType(), 
                                                  _wcsconv->physicalRefPixel().x(),
                                                  _wcsconv->physicalRefCoords().x(),
                                                  abs(_wcsconv->xWidth() * deg2pix),
                                                  _wcsconv->yType(),
                                                  _wcsconv->physicalRefPixel().y(),
                                                  _wcsconv->physicalRefCoords().y(),
                                                  abs(_wcsconv->yWidth()  * deg2pix)
                                                 );
       _wcsconv = newwcsconv;
    }
}

/* Destructor */
DetmapPixelIterator::~DetmapPixelIterator()
{
  PROC;
  
  // Allow parent to close down  
  ENDPROC;
}

/* Overridden methods */
PixelElement DetmapPixelIterator::currentItem()
{
  PROC;

  assert(_firstcalled);

  PixelElement result (_xindex,   // Do we care about xindex, yindex ?????
		       _yindex,
		       _outDetCoord[_index].x(),
		       _outDetCoord[_index].y(),
		       _wcsconv->xWidth(),
		       _wcsconv->yWidth(),
		       _valuev[_index]);

  //cout<<"DMP_CI: "<<_index<<" "<<_outDetCoord[_index].x()<<endl;

  RETURN result;
}

/* Due to possible rotation, calculate min detector coords */
CoordPair DetmapPixelIterator::physXYmin() const
{
  PROC;
  RETURN CoordPair(_minx,_miny);
}

/* Due to possible rotation, calculate max detector coords */
CoordPair DetmapPixelIterator::physXYmax() const
{
  PROC;
  RETURN CoordPair(_maxx,_maxy);
}

