// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// Module: OffCCDPSFCorrector
// Author: Richard Saxton
//
// Description: 
//
// Calculates the number of pixels in the source area which lie
// outside the live CCD area
//
// Modifications: See ChangeLog
// 
// $Id: OffCCDPSFCorrector.cc,v 1.8 2012/03/29 13:14:00 rsaxton Exp $

/* General */
#include <cmath>

/* SAS-specific */
#include "Tracer.h"
#include "DataFilter.h"
#include "Xmm.h"

/* Local */
#include "NumericalRange.h"
#include "ArfDataServer.h"
#include "OffCCDPSFCorrector.h"
#include "OffCCDPixelIterator.h"
#include "DetmapFilter.h"
#include "MissingPixCorrector.h"
#include "PixelIterator.h"

/* Constructor */
OffCCDPSFCorrector::OffCCDPSFCorrector( unsigned eefactor, PsfModel * psf):
  MissingPixCorrector(eefactor),
  _psf(psf)
{
  PROC;
  ENDPROC;
}

/* Destructor */
OffCCDPSFCorrector::~OffCCDPSFCorrector()
{
  PROC;
  ENDPROC;
}

double OffCCDPSFCorrector::eeFraction( double energy  ) const
{
  PROC;

  

  _psf->energy(energy);

 /* RDS - comment out old method 
  double fracenclosed   = 0;
  double contrib = 0;
  FOREACH(_pxitr)
    {
      PixelElement pixel = _pxitr->currentItem();
      
      
      contrib = _psf->int_psf( pixel.xCentre(),
			      pixel.yCentre(),
			      pixel.xWidth(),
			      pixel.yWidth());
      
      errstr << "OffCCDPSFCorrector: Pixel details  " << pixel << endl
	     << " PSF contribution " << contrib << endl
	     << Sas::message(Msg::App, Msg::Noisy);
      
      fracenclosed += contrib;
    }
 */
  
 // RDS - new method
  double fracenclosed2 = _psf->enclosed_psf(_pxitr);

  errstr << "OffCCDPSFCorrector: Fraction of psf enclosed " << fracenclosed2
	 << Sas::message(Msg::App, Msg::Verbose); 

  RETURN fracenclosed2;
}
  
/*
   Method: filterMap()
         : Get a pixel iterator for the Off CCD pixels
*/
void OffCCDPSFCorrector::filterMap()
{
  PROC;

  /* Create an iterator for the pixels which lie within the source region
     but off the chips or outside the observing window. */ 

          _pxitr = new OffCCDPixelIterator(
                   arfDataServer->filteredBSPixelIt(),
                   arfDataServer->positionUtils(),
                   arfDataServer->eventFlags(),
                   arfDataServer->ignoreOutofFOV(),
                   arfDataServer->ccdMask());

  ENDPROC;
}

int OffCCDPSFCorrector::totalPixels() const
{
  PROC;

   /* Calculate the total number of detector pixels covered by the bad 
      pixels */

     int total = 0;

     FOREACH(_pxitr)
      {
       PixelElement pixel = _pxitr->currentItem();

       total += pixel.xWidth() * pixel.yWidth();
      }


           errstr<<"Widths: "<<_pxitr->wcsconvertor()->xWidth()<<" "
                   <<_pxitr->wcsconvertor()->yWidth()
                   <<" tot for this CCD: "<<total
                   << Sas::message(Msg::App, Msg::Verbose);

  RETURN total;
}

