// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// Module: PsfModel
// Author: Hassan Siddiqui
//
// Description: 
//
// Abstract representation of a PSF model.
// Allows one to test either a CalPsfModel or a DummyPsfModel
// without affecting any other code.
//
// Modifications: See ChangeLog
// 
// $Id: PsfModel.h,v 1.5 2012/03/29 13:14:00 rsaxton Exp $

#ifndef PSFMODEL_H
#define PSFMODEL_H

/* SAS-specific classes */
#include "Region.h"
#include "PixelIterator.h"

class PsfModel
{
public:

  /* Virtual Destructor */
  virtual ~PsfModel(){};

  /*** NOTE: units for both x/y directions are 0.05 arcseconds ***/

  /* Return normalized PSF function at given position */
  virtual double psf( double detx, 
		      double dety) const = 0;
  
  /* Calculate integrated PSF over a square area. 
     The position of the centre
     * of the square is defined by the parameters centx and centy. */
  virtual double int_psf( double detxboxcentre, 
			  double detyboxcentre, 
			  double detxboxsize,
			  double detyboxsize) const = 0;

  virtual double enclosed_psf( PixelIterator * pxitr ) const = 0;

  virtual double rad_psf() const = 0;  // Circular only

  /* Set the appropriate energy */
  virtual void energy(double energy) = 0;

  /* Return the PSF mode */
  virtual string mode() const = 0;

  /* Set the PSF radius */
  virtual void setRadius(vector<double> radii) = 0;

  /* Set the number of grid elements */
  virtual void setNgridEls(unsigned int ngridels) = 0;

  /* Set the box shape */
  virtual void setBoxShape(ShapeBasics::ShapeType shape) = 0;

  /* Set the fast mode factor for incomplete RAW-Y extraction */
  virtual void setFastModeFactor(double factor) = 0;

  /* Set the SOURCE position in RAW-Y coords for FAST mode obs. */
  virtual void setSRCPOS(int SRCPOS) = 0;

  /* Set the orientation of an elliptical PSF model */
  virtual void setOrientation(double orient) = 0;

  /* Get the length of the semi-major axis from the elliptical PSF model */
  virtual double getMajorAxis(double x, double y) const = 0;

};

#endif
